<?php

declare(strict_types=1);

/**
 * This file is part of the MultiFlexi package
 *
 * https://github.com/VitexSoftware/php-vitexsoftware-rbczpremiumapi
 *
 * (c) Vítězslav Dvořák <http://vitexsoftware.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * Production.
 *
 * ##### API Overview - Accounts list and balance - Transaction overview (also for saving accounts) - Payments import - Statement list and download - FX rates  ##### Authentication Before making a call to Premium API, you need to register your app at our _Developer portal_. This is where you get the **ClientID** that your application must send in the request as `X-IBM-Client-Id`. This is the key that grants your app access to the API.  However, this may not be enough. Your application needs to use mTLS to call most operations here. Thus, you not only need _https_ but also a client certificate issued by us. The exception is two operations for FX rates that are accessible also without a client certificate.  Each bank client/user can issue several certificates. Each certificate can permit different sets of operations (http methods) on different bank accounts. All this must be configured in Internet Banking first by each bank client/user (bank clients need to look under _Settings_ and do not forget to download the certificate at the last step). The certificate is downloaded in **PKCS#12** format as **\\*.p12** file and protected by a password chosen by the bank client/user. Yes, your app needs the password as well to get use of the **\\*p12** file for establishing mTLS connection to the bank.  Client certificates issued in Internet Banking for bank clients/users have limited validity (e.g. **5 years**). However, **each year** certificates are automatically blocked and bank client/user must unblock them in Internet Banking. It is possible to do it in advance and prolong the time before the certificate is blocked. Your app should be prepared for these scenarios and it should communicate such cases to your user in advance to provide seamless service and high user-experience of your app.  ##### Rate Limiting The number of requests in each API operation is limited to 10 per client per sliding second and 5000 per client per sliding day. The exception is the 'Download Statement' operation with the limits lowered to 5 per client per sliding second and 1500 per client per sliding day. This is because it transports potentially sizeable binary files. The consumer must be able to handle HTTP status 429 in case of exceeding these limits.  Response headers `X-RateLimit-Limit-Second` and `X-RateLimit-Limit-Day` show the actual limits configured for the specific operation. Response headers `X-RateLimit-Remaining-Second` and `X-RateLimit-Remaining-Day` are returned to help prevent the limits from being exceeded.  ##### Notes Be aware, that in certain error situations, API can return specific error structures along with 5xx status code, which is not explicitely defined below.  ##### Quick Start Client Feel free to download a <a href=\"assets/PremiumApiClient.java\" download>simple Java client</a> that gives you quick access to our API.
 *
 * The version of the OpenAPI document: 1.1.20240910
 * Contact: info@vitexsoftware.cz
 * Generated by: https://openapi-generator.tech
 * Generator version: 7.13.0
 */

/**
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

namespace VitexSoftware\Raiffeisenbank\Model;

use VitexSoftware\Raiffeisenbank\ObjectSerializer;

/**
 * ExchangeRate Class Doc Comment.
 *
 * @category Class
 *
 * @author   OpenAPI Generator team
 *
 * @see     https://openapi-generator.tech
 *
 * @implements \ArrayAccess<string, mixed>
 */
class ExchangeRate implements \ArrayAccess, \JsonSerializable, ModelInterface
{
    public const DISCRIMINATOR = null;

    /**
     * The original name of the model.
     */
    protected static string $openAPIModelName = 'ExchangeRate';

    /**
     * Array of property to type mappings. Used for (de)serialization.
     *
     * @var string[]
     */
    protected static array $openAPITypes = [
        'countryFlagPath' => 'string',
        'currencyFrom' => 'string',
        'currencyTo' => 'string',
        'exchangeRateBuy' => 'float',
        'exchangeRateBuyCash' => 'float',
        'exchangeRateCenter' => 'float',
        'exchangeRateCenterChange' => 'float',
        'exchangeRateSell' => 'float',
        'exchangeRateSellCash' => 'float',
        'exchangeRateSellCenter' => 'float',
        'exchangeRateSellCenterPrevious' => 'float',
        'exchangeRateEcbRate' => 'float',
        'exchangeRateEcbVariation' => 'float',
        'fixedCountryCode' => 'string',
        'fixedCountryName' => 'string',
        'quotationType' => 'string',
        'unitsFrom' => 'int',
        'variableCountryCode' => 'string',
        'variableCountryName' => 'string',
    ];

    /**
     * Array of property to format mappings. Used for (de)serialization.
     *
     * @var string[]
     *
     * @phpstan-var array<string, string|null>
     *
     * @psalm-var array<string, string|null>
     */
    protected static array $openAPIFormats = [
        'countryFlagPath' => null,
        'currencyFrom' => null,
        'currencyTo' => null,
        'exchangeRateBuy' => 'double',
        'exchangeRateBuyCash' => 'double',
        'exchangeRateCenter' => 'double',
        'exchangeRateCenterChange' => 'double',
        'exchangeRateSell' => 'double',
        'exchangeRateSellCash' => 'double',
        'exchangeRateSellCenter' => 'double',
        'exchangeRateSellCenterPrevious' => 'double',
        'exchangeRateEcbRate' => 'double',
        'exchangeRateEcbVariation' => 'double',
        'fixedCountryCode' => null,
        'fixedCountryName' => null,
        'quotationType' => null,
        'unitsFrom' => 'int32',
        'variableCountryCode' => null,
        'variableCountryName' => null,
    ];

    /**
     * Array of nullable properties. Used for (de)serialization.
     *
     * @var bool[]
     */
    protected static array $openAPINullables = [
        'countryFlagPath' => false,
        'currencyFrom' => false,
        'currencyTo' => false,
        'exchangeRateBuy' => false,
        'exchangeRateBuyCash' => false,
        'exchangeRateCenter' => false,
        'exchangeRateCenterChange' => false,
        'exchangeRateSell' => false,
        'exchangeRateSellCash' => false,
        'exchangeRateSellCenter' => false,
        'exchangeRateSellCenterPrevious' => false,
        'exchangeRateEcbRate' => false,
        'exchangeRateEcbVariation' => false,
        'fixedCountryCode' => false,
        'fixedCountryName' => false,
        'quotationType' => false,
        'unitsFrom' => false,
        'variableCountryCode' => false,
        'variableCountryName' => false,
    ];

    /**
     * If a nullable field gets set to null, insert it here.
     *
     * @var bool[]
     */
    protected array $openAPINullablesSetToNull = [];

    /**
     * Array of attributes where the key is the local name,
     * and the value is the original name.
     *
     * @var string[]
     */
    protected static array $attributeMap = [
        'countryFlagPath' => 'countryFlagPath',
        'currencyFrom' => 'currencyFrom',
        'currencyTo' => 'currencyTo',
        'exchangeRateBuy' => 'exchangeRateBuy',
        'exchangeRateBuyCash' => 'exchangeRateBuyCash',
        'exchangeRateCenter' => 'exchangeRateCenter',
        'exchangeRateCenterChange' => 'exchangeRateCenterChange',
        'exchangeRateSell' => 'exchangeRateSell',
        'exchangeRateSellCash' => 'exchangeRateSellCash',
        'exchangeRateSellCenter' => 'exchangeRateSellCenter',
        'exchangeRateSellCenterPrevious' => 'exchangeRateSellCenterPrevious',
        'exchangeRateEcbRate' => 'exchangeRateEcbRate',
        'exchangeRateEcbVariation' => 'exchangeRateEcbVariation',
        'fixedCountryCode' => 'fixedCountryCode',
        'fixedCountryName' => 'fixedCountryName',
        'quotationType' => 'quotationType',
        'unitsFrom' => 'unitsFrom',
        'variableCountryCode' => 'variableCountryCode',
        'variableCountryName' => 'variableCountryName',
    ];

    /**
     * Array of attributes to setter functions (for deserialization of responses).
     *
     * @var string[]
     */
    protected static array $setters = [
        'countryFlagPath' => 'setCountryFlagPath',
        'currencyFrom' => 'setCurrencyFrom',
        'currencyTo' => 'setCurrencyTo',
        'exchangeRateBuy' => 'setExchangeRateBuy',
        'exchangeRateBuyCash' => 'setExchangeRateBuyCash',
        'exchangeRateCenter' => 'setExchangeRateCenter',
        'exchangeRateCenterChange' => 'setExchangeRateCenterChange',
        'exchangeRateSell' => 'setExchangeRateSell',
        'exchangeRateSellCash' => 'setExchangeRateSellCash',
        'exchangeRateSellCenter' => 'setExchangeRateSellCenter',
        'exchangeRateSellCenterPrevious' => 'setExchangeRateSellCenterPrevious',
        'exchangeRateEcbRate' => 'setExchangeRateEcbRate',
        'exchangeRateEcbVariation' => 'setExchangeRateEcbVariation',
        'fixedCountryCode' => 'setFixedCountryCode',
        'fixedCountryName' => 'setFixedCountryName',
        'quotationType' => 'setQuotationType',
        'unitsFrom' => 'setUnitsFrom',
        'variableCountryCode' => 'setVariableCountryCode',
        'variableCountryName' => 'setVariableCountryName',
    ];

    /**
     * Array of attributes to getter functions (for serialization of requests).
     *
     * @var string[]
     */
    protected static array $getters = [
        'countryFlagPath' => 'getCountryFlagPath',
        'currencyFrom' => 'getCurrencyFrom',
        'currencyTo' => 'getCurrencyTo',
        'exchangeRateBuy' => 'getExchangeRateBuy',
        'exchangeRateBuyCash' => 'getExchangeRateBuyCash',
        'exchangeRateCenter' => 'getExchangeRateCenter',
        'exchangeRateCenterChange' => 'getExchangeRateCenterChange',
        'exchangeRateSell' => 'getExchangeRateSell',
        'exchangeRateSellCash' => 'getExchangeRateSellCash',
        'exchangeRateSellCenter' => 'getExchangeRateSellCenter',
        'exchangeRateSellCenterPrevious' => 'getExchangeRateSellCenterPrevious',
        'exchangeRateEcbRate' => 'getExchangeRateEcbRate',
        'exchangeRateEcbVariation' => 'getExchangeRateEcbVariation',
        'fixedCountryCode' => 'getFixedCountryCode',
        'fixedCountryName' => 'getFixedCountryName',
        'quotationType' => 'getQuotationType',
        'unitsFrom' => 'getUnitsFrom',
        'variableCountryCode' => 'getVariableCountryCode',
        'variableCountryName' => 'getVariableCountryName',
    ];

    /**
     * Associative array for storing property values.
     *
     * @var mixed[]
     */
    protected array $container = [];

    /**
     * Constructor.
     *
     * @param null|mixed[] $data Associated array of property values
     *                           initializing the model
     */
    public function __construct(?array $data = null)
    {
        $this->setIfExists('countryFlagPath', $data ?? [], null);
        $this->setIfExists('currencyFrom', $data ?? [], null);
        $this->setIfExists('currencyTo', $data ?? [], null);
        $this->setIfExists('exchangeRateBuy', $data ?? [], null);
        $this->setIfExists('exchangeRateBuyCash', $data ?? [], null);
        $this->setIfExists('exchangeRateCenter', $data ?? [], null);
        $this->setIfExists('exchangeRateCenterChange', $data ?? [], null);
        $this->setIfExists('exchangeRateSell', $data ?? [], null);
        $this->setIfExists('exchangeRateSellCash', $data ?? [], null);
        $this->setIfExists('exchangeRateSellCenter', $data ?? [], null);
        $this->setIfExists('exchangeRateSellCenterPrevious', $data ?? [], null);
        $this->setIfExists('exchangeRateEcbRate', $data ?? [], null);
        $this->setIfExists('exchangeRateEcbVariation', $data ?? [], null);
        $this->setIfExists('fixedCountryCode', $data ?? [], null);
        $this->setIfExists('fixedCountryName', $data ?? [], null);
        $this->setIfExists('quotationType', $data ?? [], null);
        $this->setIfExists('unitsFrom', $data ?? [], null);
        $this->setIfExists('variableCountryCode', $data ?? [], null);
        $this->setIfExists('variableCountryName', $data ?? [], null);
    }

    /**
     * Gets the string presentation of the object.
     *
     * @return string
     */
    public function __toString()
    {
        return json_encode(
            ObjectSerializer::sanitizeForSerialization($this),
            \JSON_PRETTY_PRINT,
        );
    }

    /**
     * Array of property to type mappings. Used for (de)serialization.
     *
     * @return array
     */
    public static function openAPITypes()
    {
        return self::$openAPITypes;
    }

    /**
     * Array of property to format mappings. Used for (de)serialization.
     *
     * @return array
     */
    public static function openAPIFormats()
    {
        return self::$openAPIFormats;
    }

    /**
     * Checks if a property is nullable.
     */
    public static function isNullable(string $property): bool
    {
        return self::openAPINullables()[$property] ?? false;
    }

    /**
     * Checks if a nullable property is set to null.
     */
    public function isNullableSetToNull(string $property): bool
    {
        return \in_array($property, $this->getOpenAPINullablesSetToNull(), true);
    }

    /**
     * Array of attributes where the key is the local name,
     * and the value is the original name.
     *
     * @return array
     */
    public static function attributeMap()
    {
        return self::$attributeMap;
    }

    /**
     * Array of attributes to setter functions (for deserialization of responses).
     *
     * @return array
     */
    public static function setters()
    {
        return self::$setters;
    }

    /**
     * Array of attributes to getter functions (for serialization of requests).
     *
     * @return array
     */
    public static function getters()
    {
        return self::$getters;
    }

    /**
     * The original name of the model.
     *
     * @return string
     */
    public function getModelName()
    {
        return self::$openAPIModelName;
    }

    /**
     * Show all the invalid properties with reasons.
     *
     * @return array invalid properties with reasons
     */
    public function listInvalidProperties()
    {
        $invalidProperties = [];

        if ($this->container['currencyFrom'] === null) {
            $invalidProperties[] = "'currencyFrom' can't be null";
        }

        if (mb_strlen((string) $this->container['currencyFrom']) > 3) {
            $invalidProperties[] = "invalid value for 'currencyFrom', the character length must be smaller than or equal to 3.";
        }

        if (mb_strlen((string) $this->container['currencyFrom']) < 3) {
            $invalidProperties[] = "invalid value for 'currencyFrom', the character length must be bigger than or equal to 3.";
        }

        if ($this->container['currencyTo'] === null) {
            $invalidProperties[] = "'currencyTo' can't be null";
        }

        if (mb_strlen((string) $this->container['currencyTo']) > 3) {
            $invalidProperties[] = "invalid value for 'currencyTo', the character length must be smaller than or equal to 3.";
        }

        if (mb_strlen((string) $this->container['currencyTo']) < 3) {
            $invalidProperties[] = "invalid value for 'currencyTo', the character length must be bigger than or equal to 3.";
        }

        if ($this->container['exchangeRateBuy'] === null) {
            $invalidProperties[] = "'exchangeRateBuy' can't be null";
        }

        if ($this->container['exchangeRateBuyCash'] === null) {
            $invalidProperties[] = "'exchangeRateBuyCash' can't be null";
        }

        if ($this->container['exchangeRateCenter'] === null) {
            $invalidProperties[] = "'exchangeRateCenter' can't be null";
        }

        if ($this->container['exchangeRateCenterChange'] === null) {
            $invalidProperties[] = "'exchangeRateCenterChange' can't be null";
        }

        if ($this->container['exchangeRateSell'] === null) {
            $invalidProperties[] = "'exchangeRateSell' can't be null";
        }

        if ($this->container['exchangeRateSellCash'] === null) {
            $invalidProperties[] = "'exchangeRateSellCash' can't be null";
        }

        if ($this->container['exchangeRateSellCenter'] === null) {
            $invalidProperties[] = "'exchangeRateSellCenter' can't be null";
        }

        if ($this->container['exchangeRateSellCenterPrevious'] === null) {
            $invalidProperties[] = "'exchangeRateSellCenterPrevious' can't be null";
        }

        if ($this->container['quotationType'] === null) {
            $invalidProperties[] = "'quotationType' can't be null";
        }

        if ($this->container['unitsFrom'] === null) {
            $invalidProperties[] = "'unitsFrom' can't be null";
        }

        return $invalidProperties;
    }

    /**
     * Validate all the properties in the model
     * return true if all passed.
     *
     * @return bool True if all properties are valid
     */
    public function valid()
    {
        return \count($this->listInvalidProperties()) === 0;
    }

    /**
     * Gets countryFlagPath.
     *
     * @return null|string
     */
    public function getCountryFlagPath()
    {
        return $this->container['countryFlagPath'];
    }

    /**
     * Sets countryFlagPath.
     *
     * @param null|string $countryFlagPath countryFlagPath
     *
     * @return self
     */
    public function setCountryFlagPath($countryFlagPath)
    {
        if (null === $countryFlagPath) {
            throw new \InvalidArgumentException('non-nullable countryFlagPath cannot be null');
        }

        $this->container['countryFlagPath'] = $countryFlagPath;

        return $this;
    }

    /**
     * Gets currencyFrom.
     *
     * @return string
     */
    public function getCurrencyFrom()
    {
        return $this->container['currencyFrom'];
    }

    /**
     * Sets currencyFrom.
     *
     * @param string $currencyFrom currencyFrom
     *
     * @return self
     */
    public function setCurrencyFrom($currencyFrom)
    {
        if (null === $currencyFrom) {
            throw new \InvalidArgumentException('non-nullable currencyFrom cannot be null');
        }

        if (mb_strlen((string) $currencyFrom) > 3) {
            throw new \InvalidArgumentException('invalid length for $currencyFrom when calling ExchangeRate., must be smaller than or equal to 3.');
        }

        if (mb_strlen((string) $currencyFrom) < 3) {
            throw new \InvalidArgumentException('invalid length for $currencyFrom when calling ExchangeRate., must be bigger than or equal to 3.');
        }

        $this->container['currencyFrom'] = $currencyFrom;

        return $this;
    }

    /**
     * Gets currencyTo.
     *
     * @return string
     */
    public function getCurrencyTo()
    {
        return $this->container['currencyTo'];
    }

    /**
     * Sets currencyTo.
     *
     * @param string $currencyTo currencyTo
     *
     * @return self
     */
    public function setCurrencyTo($currencyTo)
    {
        if (null === $currencyTo) {
            throw new \InvalidArgumentException('non-nullable currencyTo cannot be null');
        }

        if (mb_strlen((string) $currencyTo) > 3) {
            throw new \InvalidArgumentException('invalid length for $currencyTo when calling ExchangeRate., must be smaller than or equal to 3.');
        }

        if (mb_strlen((string) $currencyTo) < 3) {
            throw new \InvalidArgumentException('invalid length for $currencyTo when calling ExchangeRate., must be bigger than or equal to 3.');
        }

        $this->container['currencyTo'] = $currencyTo;

        return $this;
    }

    /**
     * Gets exchangeRateBuy.
     *
     * @return float
     */
    public function getExchangeRateBuy()
    {
        return $this->container['exchangeRateBuy'];
    }

    /**
     * Sets exchangeRateBuy.
     *
     * @param float $exchangeRateBuy exchangeRateBuy
     *
     * @return self
     */
    public function setExchangeRateBuy($exchangeRateBuy)
    {
        if (null === $exchangeRateBuy) {
            throw new \InvalidArgumentException('non-nullable exchangeRateBuy cannot be null');
        }

        $this->container['exchangeRateBuy'] = $exchangeRateBuy;

        return $this;
    }

    /**
     * Gets exchangeRateBuyCash.
     *
     * @return float
     */
    public function getExchangeRateBuyCash()
    {
        return $this->container['exchangeRateBuyCash'];
    }

    /**
     * Sets exchangeRateBuyCash.
     *
     * @param float $exchangeRateBuyCash exchangeRateBuyCash
     *
     * @return self
     */
    public function setExchangeRateBuyCash($exchangeRateBuyCash)
    {
        if (null === $exchangeRateBuyCash) {
            throw new \InvalidArgumentException('non-nullable exchangeRateBuyCash cannot be null');
        }

        $this->container['exchangeRateBuyCash'] = $exchangeRateBuyCash;

        return $this;
    }

    /**
     * Gets exchangeRateCenter.
     *
     * @return float
     */
    public function getExchangeRateCenter()
    {
        return $this->container['exchangeRateCenter'];
    }

    /**
     * Sets exchangeRateCenter.
     *
     * @param float $exchangeRateCenter exchangeRateCenter
     *
     * @return self
     */
    public function setExchangeRateCenter($exchangeRateCenter)
    {
        if (null === $exchangeRateCenter) {
            throw new \InvalidArgumentException('non-nullable exchangeRateCenter cannot be null');
        }

        $this->container['exchangeRateCenter'] = $exchangeRateCenter;

        return $this;
    }

    /**
     * Gets exchangeRateCenterChange.
     *
     * @return float
     */
    public function getExchangeRateCenterChange()
    {
        return $this->container['exchangeRateCenterChange'];
    }

    /**
     * Sets exchangeRateCenterChange.
     *
     * @param float $exchangeRateCenterChange exchangeRateCenterChange
     *
     * @return self
     */
    public function setExchangeRateCenterChange($exchangeRateCenterChange)
    {
        if (null === $exchangeRateCenterChange) {
            throw new \InvalidArgumentException('non-nullable exchangeRateCenterChange cannot be null');
        }

        $this->container['exchangeRateCenterChange'] = $exchangeRateCenterChange;

        return $this;
    }

    /**
     * Gets exchangeRateSell.
     *
     * @return float
     */
    public function getExchangeRateSell()
    {
        return $this->container['exchangeRateSell'];
    }

    /**
     * Sets exchangeRateSell.
     *
     * @param float $exchangeRateSell exchangeRateSell
     *
     * @return self
     */
    public function setExchangeRateSell($exchangeRateSell)
    {
        if (null === $exchangeRateSell) {
            throw new \InvalidArgumentException('non-nullable exchangeRateSell cannot be null');
        }

        $this->container['exchangeRateSell'] = $exchangeRateSell;

        return $this;
    }

    /**
     * Gets exchangeRateSellCash.
     *
     * @return float
     */
    public function getExchangeRateSellCash()
    {
        return $this->container['exchangeRateSellCash'];
    }

    /**
     * Sets exchangeRateSellCash.
     *
     * @param float $exchangeRateSellCash exchangeRateSellCash
     *
     * @return self
     */
    public function setExchangeRateSellCash($exchangeRateSellCash)
    {
        if (null === $exchangeRateSellCash) {
            throw new \InvalidArgumentException('non-nullable exchangeRateSellCash cannot be null');
        }

        $this->container['exchangeRateSellCash'] = $exchangeRateSellCash;

        return $this;
    }

    /**
     * Gets exchangeRateSellCenter.
     *
     * @return float
     */
    public function getExchangeRateSellCenter()
    {
        return $this->container['exchangeRateSellCenter'];
    }

    /**
     * Sets exchangeRateSellCenter.
     *
     * @param float $exchangeRateSellCenter exchangeRateSellCenter
     *
     * @return self
     */
    public function setExchangeRateSellCenter($exchangeRateSellCenter)
    {
        if (null === $exchangeRateSellCenter) {
            throw new \InvalidArgumentException('non-nullable exchangeRateSellCenter cannot be null');
        }

        $this->container['exchangeRateSellCenter'] = $exchangeRateSellCenter;

        return $this;
    }

    /**
     * Gets exchangeRateSellCenterPrevious.
     *
     * @return float
     */
    public function getExchangeRateSellCenterPrevious()
    {
        return $this->container['exchangeRateSellCenterPrevious'];
    }

    /**
     * Sets exchangeRateSellCenterPrevious.
     *
     * @param float $exchangeRateSellCenterPrevious exchangeRateSellCenterPrevious
     *
     * @return self
     */
    public function setExchangeRateSellCenterPrevious($exchangeRateSellCenterPrevious)
    {
        if (null === $exchangeRateSellCenterPrevious) {
            throw new \InvalidArgumentException('non-nullable exchangeRateSellCenterPrevious cannot be null');
        }

        $this->container['exchangeRateSellCenterPrevious'] = $exchangeRateSellCenterPrevious;

        return $this;
    }

    /**
     * Gets exchangeRateEcbRate.
     *
     * @return null|float
     */
    public function getExchangeRateEcbRate()
    {
        return $this->container['exchangeRateEcbRate'];
    }

    /**
     * Sets exchangeRateEcbRate.
     *
     * @param null|float $exchangeRateEcbRate exchangeRateEcbRate
     *
     * @return self
     */
    public function setExchangeRateEcbRate($exchangeRateEcbRate)
    {
        if (null === $exchangeRateEcbRate) {
            throw new \InvalidArgumentException('non-nullable exchangeRateEcbRate cannot be null');
        }

        $this->container['exchangeRateEcbRate'] = $exchangeRateEcbRate;

        return $this;
    }

    /**
     * Gets exchangeRateEcbVariation.
     *
     * @return null|float
     */
    public function getExchangeRateEcbVariation()
    {
        return $this->container['exchangeRateEcbVariation'];
    }

    /**
     * Sets exchangeRateEcbVariation.
     *
     * @param null|float $exchangeRateEcbVariation exchangeRateEcbVariation
     *
     * @return self
     */
    public function setExchangeRateEcbVariation($exchangeRateEcbVariation)
    {
        if (null === $exchangeRateEcbVariation) {
            throw new \InvalidArgumentException('non-nullable exchangeRateEcbVariation cannot be null');
        }

        $this->container['exchangeRateEcbVariation'] = $exchangeRateEcbVariation;

        return $this;
    }

    /**
     * Gets fixedCountryCode.
     *
     * @return null|string
     */
    public function getFixedCountryCode()
    {
        return $this->container['fixedCountryCode'];
    }

    /**
     * Sets fixedCountryCode.
     *
     * @param null|string $fixedCountryCode fixedCountryCode
     *
     * @return self
     */
    public function setFixedCountryCode($fixedCountryCode)
    {
        if (null === $fixedCountryCode) {
            throw new \InvalidArgumentException('non-nullable fixedCountryCode cannot be null');
        }

        $this->container['fixedCountryCode'] = $fixedCountryCode;

        return $this;
    }

    /**
     * Gets fixedCountryName.
     *
     * @return null|string
     */
    public function getFixedCountryName()
    {
        return $this->container['fixedCountryName'];
    }

    /**
     * Sets fixedCountryName.
     *
     * @param null|string $fixedCountryName fixedCountryName
     *
     * @return self
     */
    public function setFixedCountryName($fixedCountryName)
    {
        if (null === $fixedCountryName) {
            throw new \InvalidArgumentException('non-nullable fixedCountryName cannot be null');
        }

        $this->container['fixedCountryName'] = $fixedCountryName;

        return $this;
    }

    /**
     * Gets quotationType.
     *
     * @return string
     */
    public function getQuotationType()
    {
        return $this->container['quotationType'];
    }

    /**
     * Sets quotationType.
     *
     * @param string $quotationType quotationType
     *
     * @return self
     */
    public function setQuotationType($quotationType)
    {
        if (null === $quotationType) {
            throw new \InvalidArgumentException('non-nullable quotationType cannot be null');
        }

        $this->container['quotationType'] = $quotationType;

        return $this;
    }

    /**
     * Gets unitsFrom.
     *
     * @return int
     */
    public function getUnitsFrom()
    {
        return $this->container['unitsFrom'];
    }

    /**
     * Sets unitsFrom.
     *
     * @param int $unitsFrom unitsFrom
     *
     * @return self
     */
    public function setUnitsFrom($unitsFrom)
    {
        if (null === $unitsFrom) {
            throw new \InvalidArgumentException('non-nullable unitsFrom cannot be null');
        }

        $this->container['unitsFrom'] = $unitsFrom;

        return $this;
    }

    /**
     * Gets variableCountryCode.
     *
     * @return null|string
     */
    public function getVariableCountryCode()
    {
        return $this->container['variableCountryCode'];
    }

    /**
     * Sets variableCountryCode.
     *
     * @param null|string $variableCountryCode variableCountryCode
     *
     * @return self
     */
    public function setVariableCountryCode($variableCountryCode)
    {
        if (null === $variableCountryCode) {
            throw new \InvalidArgumentException('non-nullable variableCountryCode cannot be null');
        }

        $this->container['variableCountryCode'] = $variableCountryCode;

        return $this;
    }

    /**
     * Gets variableCountryName.
     *
     * @return null|string
     */
    public function getVariableCountryName()
    {
        return $this->container['variableCountryName'];
    }

    /**
     * Sets variableCountryName.
     *
     * @param null|string $variableCountryName variableCountryName
     *
     * @return self
     */
    public function setVariableCountryName($variableCountryName)
    {
        if (null === $variableCountryName) {
            throw new \InvalidArgumentException('non-nullable variableCountryName cannot be null');
        }

        $this->container['variableCountryName'] = $variableCountryName;

        return $this;
    }
    /**
     * Returns true if offset exists. False otherwise.
     *
     * @param int $offset Offset
     */
    public function offsetExists($offset): bool
    {
        return isset($this->container[$offset]);
    }

    /**
     * Gets offset.
     *
     * @param int $offset Offset
     *
     * @return null|mixed
     */
    #[\ReturnTypeWillChange]
    public function offsetGet($offset)
    {
        return $this->container[$offset] ?? null;
    }

    /**
     * Sets value based on offset.
     *
     * @param null|int $offset Offset
     * @param mixed    $value  Value to be set
     */
    public function offsetSet($offset, $value): void
    {
        if (null === $offset) {
            $this->container[] = $value;
        } else {
            $this->container[$offset] = $value;
        }
    }

    /**
     * Unsets offset.
     *
     * @param int $offset Offset
     */
    public function offsetUnset($offset): void
    {
        unset($this->container[$offset]);
    }

    /**
     * Serializes the object to a value that can be serialized natively by json_encode().
     *
     * @see https://www.php.net/manual/en/jsonserializable.jsonserialize.php
     *
     * @return mixed returns data which can be serialized by json_encode(), which is a value
     *               of any type other than a resource
     */
    #[\ReturnTypeWillChange]
    public function jsonSerialize()
    {
        return ObjectSerializer::sanitizeForSerialization($this);
    }

    /**
     * Gets a header-safe presentation of the object.
     *
     * @return string
     */
    public function toHeaderValue()
    {
        return json_encode(ObjectSerializer::sanitizeForSerialization($this));
    }

    /**
     * Array of nullable properties.
     */
    protected static function openAPINullables(): array
    {
        return self::$openAPINullables;
    }

    /**
     * Array of nullable field names deliberately set to null.
     *
     * @return bool[]
     */
    private function getOpenAPINullablesSetToNull(): array
    {
        return $this->openAPINullablesSetToNull;
    }

    /**
     * Setter - Array of nullable field names deliberately set to null.
     *
     * @param bool[] $openAPINullablesSetToNull
     */
    private function setOpenAPINullablesSetToNull(array $openAPINullablesSetToNull): void
    {
        $this->openAPINullablesSetToNull = $openAPINullablesSetToNull;
    }

    /**
     * Sets $this->container[$variableName] to the given data or to the given default Value; if $variableName
     * is nullable and its value is set to null in the $fields array, then mark it as "set to null" in the
     * $this->openAPINullablesSetToNull array.
     *
     * @param mixed $defaultValue
     */
    private function setIfExists(string $variableName, array $fields, $defaultValue): void
    {
        if (self::isNullable($variableName) && \array_key_exists($variableName, $fields) && null === $fields[$variableName]) {
            $this->openAPINullablesSetToNull[] = $variableName;
        }

        $this->container[$variableName] = $fields[$variableName] ?? $defaultValue;
    }
}
