<?php

declare(strict_types=1);

/**
 * This file is part of the MultiFlexi package
 *
 * https://github.com/VitexSoftware/php-vitexsoftware-rbczpremiumapi
 *
 * (c) Vítězslav Dvořák <http://vitexsoftware.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * Production.
 *
 * ##### API Overview - Accounts list and balance - Transaction overview (also for saving accounts) - Payments import - Statement list and download - FX rates  ##### Authentication Before making a call to Premium API, you need to register your app at our _Developer portal_. This is where you get the **ClientID** that your application must send in the request as `X-IBM-Client-Id`. This is the key that grants your app access to the API.  However, this may not be enough. Your application needs to use mTLS to call most operations here. Thus, you not only need _https_ but also a client certificate issued by us. The exception is two operations for FX rates that are accessible also without a client certificate.  Each bank client/user can issue several certificates. Each certificate can permit different sets of operations (http methods) on different bank accounts. All this must be configured in Internet Banking first by each bank client/user (bank clients need to look under _Settings_ and do not forget to download the certificate at the last step). The certificate is downloaded in **PKCS#12** format as **\\*.p12** file and protected by a password chosen by the bank client/user. Yes, your app needs the password as well to get use of the **\\*p12** file for establishing mTLS connection to the bank.  Client certificates issued in Internet Banking for bank clients/users have limited validity (e.g. **5 years**). However, **each year** certificates are automatically blocked and bank client/user must unblock them in Internet Banking. It is possible to do it in advance and prolong the time before the certificate is blocked. Your app should be prepared for these scenarios and it should communicate such cases to your user in advance to provide seamless service and high user-experience of your app.  ##### Rate Limiting The number of requests in each API operation is limited to 10 per client per sliding second and 5000 per client per sliding day. The exception is the 'Download Statement' operation with the limits lowered to 5 per client per sliding second and 1500 per client per sliding day. This is because it transports potentially sizeable binary files. The consumer must be able to handle HTTP status 429 in case of exceeding these limits.  Response headers `X-RateLimit-Limit-Second` and `X-RateLimit-Limit-Day` show the actual limits configured for the specific operation. Response headers `X-RateLimit-Remaining-Second` and `X-RateLimit-Remaining-Day` are returned to help prevent the limits from being exceeded.  ##### Notes Be aware, that in certain error situations, API can return specific error structures along with 5xx status code, which is not explicitely defined below.  ##### Quick Start Client Feel free to download a <a href=\"assets/PremiumApiClient.java\" download>simple Java client</a> that gives you quick access to our API.
 *
 * The version of the OpenAPI document: 1.1.20240910
 * Contact: info@vitexsoftware.cz
 * Generated by: https://openapi-generator.tech
 * Generator version: 7.13.0
 */

/**
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

namespace VitexSoftware\Raiffeisenbank\Model;

use VitexSoftware\Raiffeisenbank\ObjectSerializer;

/**
 * GetBatchDetail200Response Class Doc Comment.
 *
 * @category Class
 *
 * @author   OpenAPI Generator team
 *
 * @see     https://openapi-generator.tech
 *
 * @implements \ArrayAccess<string, mixed>
 */
class GetBatchDetail200Response implements \ArrayAccess, \JsonSerializable, ModelInterface
{
    public const DISCRIMINATOR = null;
    public const BATCH_FILE_STATUS_DELETED = 'DELETED';
    public const BATCH_FILE_STATUS_OK = 'OK';
    public const BATCH_FILE_STATUS_UPLOADED = 'UPLOADED';
    public const BATCH_FILE_STATUS_PROCESSING = 'PROCESSING';
    public const BATCH_FILE_STATUS_ERROR = 'ERROR';

    /**
     * The original name of the model.
     */
    protected static string $openAPIModelName = 'getBatchDetail_200_response';

    /**
     * Array of property to type mappings. Used for (de)serialization.
     *
     * @var string[]
     */
    protected static array $openAPITypes = [
        'batchName' => 'string',
        'batchFileStatus' => 'string',
        'createDate' => '\DateTime',
        'batchItems' => '\VitexSoftware\Raiffeisenbank\Model\GetBatchDetail200ResponseBatchItemsInner[]',
    ];

    /**
     * Array of property to format mappings. Used for (de)serialization.
     *
     * @var string[]
     *
     * @phpstan-var array<string, string|null>
     *
     * @psalm-var array<string, string|null>
     */
    protected static array $openAPIFormats = [
        'batchName' => null,
        'batchFileStatus' => null,
        'createDate' => 'date',
        'batchItems' => null,
    ];

    /**
     * Array of nullable properties. Used for (de)serialization.
     *
     * @var bool[]
     */
    protected static array $openAPINullables = [
        'batchName' => false,
        'batchFileStatus' => false,
        'createDate' => false,
        'batchItems' => false,
    ];

    /**
     * If a nullable field gets set to null, insert it here.
     *
     * @var bool[]
     */
    protected array $openAPINullablesSetToNull = [];

    /**
     * Array of attributes where the key is the local name,
     * and the value is the original name.
     *
     * @var string[]
     */
    protected static array $attributeMap = [
        'batchName' => 'batchName',
        'batchFileStatus' => 'batchFileStatus',
        'createDate' => 'createDate',
        'batchItems' => 'batchItems',
    ];

    /**
     * Array of attributes to setter functions (for deserialization of responses).
     *
     * @var string[]
     */
    protected static array $setters = [
        'batchName' => 'setBatchName',
        'batchFileStatus' => 'setBatchFileStatus',
        'createDate' => 'setCreateDate',
        'batchItems' => 'setBatchItems',
    ];

    /**
     * Array of attributes to getter functions (for serialization of requests).
     *
     * @var string[]
     */
    protected static array $getters = [
        'batchName' => 'getBatchName',
        'batchFileStatus' => 'getBatchFileStatus',
        'createDate' => 'getCreateDate',
        'batchItems' => 'getBatchItems',
    ];

    /**
     * Associative array for storing property values.
     *
     * @var mixed[]
     */
    protected array $container = [];

    /**
     * Constructor.
     *
     * @param null|mixed[] $data Associated array of property values
     *                           initializing the model
     */
    public function __construct(?array $data = null)
    {
        $this->setIfExists('batchName', $data ?? [], null);
        $this->setIfExists('batchFileStatus', $data ?? [], null);
        $this->setIfExists('createDate', $data ?? [], null);
        $this->setIfExists('batchItems', $data ?? [], null);
    }

    /**
     * Gets the string presentation of the object.
     *
     * @return string
     */
    public function __toString()
    {
        return json_encode(
            ObjectSerializer::sanitizeForSerialization($this),
            \JSON_PRETTY_PRINT,
        );
    }

    /**
     * Array of property to type mappings. Used for (de)serialization.
     *
     * @return array
     */
    public static function openAPITypes()
    {
        return self::$openAPITypes;
    }

    /**
     * Array of property to format mappings. Used for (de)serialization.
     *
     * @return array
     */
    public static function openAPIFormats()
    {
        return self::$openAPIFormats;
    }

    /**
     * Checks if a property is nullable.
     */
    public static function isNullable(string $property): bool
    {
        return self::openAPINullables()[$property] ?? false;
    }

    /**
     * Checks if a nullable property is set to null.
     */
    public function isNullableSetToNull(string $property): bool
    {
        return \in_array($property, $this->getOpenAPINullablesSetToNull(), true);
    }

    /**
     * Array of attributes where the key is the local name,
     * and the value is the original name.
     *
     * @return array
     */
    public static function attributeMap()
    {
        return self::$attributeMap;
    }

    /**
     * Array of attributes to setter functions (for deserialization of responses).
     *
     * @return array
     */
    public static function setters()
    {
        return self::$setters;
    }

    /**
     * Array of attributes to getter functions (for serialization of requests).
     *
     * @return array
     */
    public static function getters()
    {
        return self::$getters;
    }

    /**
     * The original name of the model.
     *
     * @return string
     */
    public function getModelName()
    {
        return self::$openAPIModelName;
    }

    /**
     * Gets allowable values of the enum.
     *
     * @return string[]
     */
    public function getBatchFileStatusAllowableValues()
    {
        return [
            self::BATCH_FILE_STATUS_DELETED,
            self::BATCH_FILE_STATUS_OK,
            self::BATCH_FILE_STATUS_UPLOADED,
            self::BATCH_FILE_STATUS_PROCESSING,
            self::BATCH_FILE_STATUS_ERROR,
        ];
    }

    /**
     * Show all the invalid properties with reasons.
     *
     * @return array invalid properties with reasons
     */
    public function listInvalidProperties()
    {
        $invalidProperties = [];

        $allowedValues = $this->getBatchFileStatusAllowableValues();

        if (null !== $this->container['batchFileStatus'] && !\in_array($this->container['batchFileStatus'], $allowedValues, true)) {
            $invalidProperties[] = sprintf(
                "invalid value '%s' for 'batchFileStatus', must be one of '%s'",
                $this->container['batchFileStatus'],
                implode("', '", $allowedValues),
            );
        }

        return $invalidProperties;
    }

    /**
     * Validate all the properties in the model
     * return true if all passed.
     *
     * @return bool True if all properties are valid
     */
    public function valid()
    {
        return \count($this->listInvalidProperties()) === 0;
    }

    /**
     * Gets batchName.
     *
     * @return null|string
     */
    public function getBatchName()
    {
        return $this->container['batchName'];
    }

    /**
     * Sets batchName.
     *
     * @param null|string $batchName Batch name
     *
     * @return self
     */
    public function setBatchName($batchName)
    {
        if (null === $batchName) {
            throw new \InvalidArgumentException('non-nullable batchName cannot be null');
        }

        $this->container['batchName'] = $batchName;

        return $this;
    }

    /**
     * Gets batchFileStatus.
     *
     * @return null|string
     */
    public function getBatchFileStatus()
    {
        return $this->container['batchFileStatus'];
    }

    /**
     * Sets batchFileStatus.
     *
     * @param null|string $batchFileStatus Status of batch file import
     *
     * @return self
     */
    public function setBatchFileStatus($batchFileStatus)
    {
        if (null === $batchFileStatus) {
            throw new \InvalidArgumentException('non-nullable batchFileStatus cannot be null');
        }

        $allowedValues = $this->getBatchFileStatusAllowableValues();

        if (!\in_array($batchFileStatus, $allowedValues, true)) {
            throw new \InvalidArgumentException(
                sprintf(
                    "Invalid value '%s' for 'batchFileStatus', must be one of '%s'",
                    $batchFileStatus,
                    implode("', '", $allowedValues),
                ),
            );
        }

        $this->container['batchFileStatus'] = $batchFileStatus;

        return $this;
    }

    /**
     * Gets createDate.
     *
     * @return null|\DateTime
     */
    public function getCreateDate()
    {
        return $this->container['createDate'];
    }

    /**
     * Sets createDate.
     *
     * @param null|\DateTime $createDate The date when the batch was created
     *
     * @return self
     */
    public function setCreateDate($createDate)
    {
        if (null === $createDate) {
            throw new \InvalidArgumentException('non-nullable createDate cannot be null');
        }

        $this->container['createDate'] = $createDate;

        return $this;
    }

    /**
     * Gets batchItems.
     *
     * @return null|\VitexSoftware\Raiffeisenbank\Model\GetBatchDetail200ResponseBatchItemsInner[]
     */
    public function getBatchItems()
    {
        return $this->container['batchItems'];
    }

    /**
     * Sets batchItems.
     *
     * @param null|\VitexSoftware\Raiffeisenbank\Model\GetBatchDetail200ResponseBatchItemsInner[] $batchItems batchItems
     *
     * @return self
     */
    public function setBatchItems($batchItems)
    {
        if (null === $batchItems) {
            throw new \InvalidArgumentException('non-nullable batchItems cannot be null');
        }

        $this->container['batchItems'] = $batchItems;

        return $this;
    }
    /**
     * Returns true if offset exists. False otherwise.
     *
     * @param int $offset Offset
     */
    public function offsetExists($offset): bool
    {
        return isset($this->container[$offset]);
    }

    /**
     * Gets offset.
     *
     * @param int $offset Offset
     *
     * @return null|mixed
     */
    #[\ReturnTypeWillChange]
    public function offsetGet($offset)
    {
        return $this->container[$offset] ?? null;
    }

    /**
     * Sets value based on offset.
     *
     * @param null|int $offset Offset
     * @param mixed    $value  Value to be set
     */
    public function offsetSet($offset, $value): void
    {
        if (null === $offset) {
            $this->container[] = $value;
        } else {
            $this->container[$offset] = $value;
        }
    }

    /**
     * Unsets offset.
     *
     * @param int $offset Offset
     */
    public function offsetUnset($offset): void
    {
        unset($this->container[$offset]);
    }

    /**
     * Serializes the object to a value that can be serialized natively by json_encode().
     *
     * @see https://www.php.net/manual/en/jsonserializable.jsonserialize.php
     *
     * @return mixed returns data which can be serialized by json_encode(), which is a value
     *               of any type other than a resource
     */
    #[\ReturnTypeWillChange]
    public function jsonSerialize()
    {
        return ObjectSerializer::sanitizeForSerialization($this);
    }

    /**
     * Gets a header-safe presentation of the object.
     *
     * @return string
     */
    public function toHeaderValue()
    {
        return json_encode(ObjectSerializer::sanitizeForSerialization($this));
    }

    /**
     * Array of nullable properties.
     */
    protected static function openAPINullables(): array
    {
        return self::$openAPINullables;
    }

    /**
     * Array of nullable field names deliberately set to null.
     *
     * @return bool[]
     */
    private function getOpenAPINullablesSetToNull(): array
    {
        return $this->openAPINullablesSetToNull;
    }

    /**
     * Setter - Array of nullable field names deliberately set to null.
     *
     * @param bool[] $openAPINullablesSetToNull
     */
    private function setOpenAPINullablesSetToNull(array $openAPINullablesSetToNull): void
    {
        $this->openAPINullablesSetToNull = $openAPINullablesSetToNull;
    }

    /**
     * Sets $this->container[$variableName] to the given data or to the given default Value; if $variableName
     * is nullable and its value is set to null in the $fields array, then mark it as "set to null" in the
     * $this->openAPINullablesSetToNull array.
     *
     * @param mixed $defaultValue
     */
    private function setIfExists(string $variableName, array $fields, $defaultValue): void
    {
        if (self::isNullable($variableName) && \array_key_exists($variableName, $fields) && null === $fields[$variableName]) {
            $this->openAPINullablesSetToNull[] = $variableName;
        }

        $this->container[$variableName] = $fields[$variableName] ?? $defaultValue;
    }
}
